﻿/**
 * @file This file contains the PageDynamic class
 * @author michael.schertler@outlook.de
 */
/**
 * @class
 * @author michael.schertler@outlook.de
 * @classdesc This class handles general features, needed on all pages
 * @description Use the constructor to pass the amount of tabs & acticle number
 * @param {Number} numberOfTabs Amount of tabs
 * @param {String} articleNumber Identification of this BTL
 */
function PageDynamic(numberOfTabs, articleNumber) {
  /**
   * @type {Number}
   * @description This holds the amount of tabs, needed within serveral cases
   */
  this.tabCount = numberOfTabs;
  /**
  * @type {Object}
  * @description This is used to identify the module
  * @property {String} moduleIdentifier.ArticleNumber - The given article number
  * @property {String} moduleIdentifier.Type - BTL Type [BTL7]
  * @property {String} moduleIdentifier.Version - BTL Version [V50T]
  * @property {String} moduleIdentifier.Network - Fieldbus type [T, D, E, L]
  * @property {String} moduleIdentifier.Length - Nominal stroke, 4 digits
  * @property {String} moduleIdentifier.Style - Rod or Profile [B, P]
  * @property {String} moduleIdentifier.Connection - Connector type [C003]
 */
  this.moduleIdentifier = { "ArticleNumber": articleNumber, "Type": articleNumber.split("-")[0], "Version": articleNumber.split("-")[1].substring(0, 3), "Network": articleNumber.split("-")[1].substring(3, 4), "Length": articleNumber.split("-")[2].substring(1, 5), "Style": articleNumber.split("-")[3], "Connection": articleNumber.split("-")[4] };
  /**
   * @description This method sets the page status (upper right corner)
   * @param {String} message Text content of status
   * @param {Boolean} isLoading If the loading animation should be displayed
   * @param {Boolean} isVisible If the status should be visible
   */
  this.setStatus = function (message, isLoading, isVisible) {
    if (isVisible) {
      document.getElementById("PageStatus").style.display = "";
      if (isLoading) document.getElementById("PageStatus").children[0].style.display = "";
      else document.getElementById("PageStatus").children[0].style.display = "none";
      document.getElementById("PageStatus").children[1].innerHTML = message;
    } else document.getElementById("PageStatus").style.display = "none";
  }
  /**
   * @description Global error fallback method, displays the error as status message and cancels the update interval
   * @param {String} event Error message / description
   */
  this.error = function (event) {
    window.dynamic.setStatus(event, false, true);
    document.getElementById("PageStatus").style.fontWeight = "bold";
    document.getElementById("PageStatus").style.color = "Red";
    document.getElementById("PageStatus").style.backgroundColor = "White";
    if (dynamic.updateInterval != null) window.clearInterval(dynamic.updateInterval);
  }
  window.onerror = this.error;
  document.onerror = this.error;
  /**
   * @description This method shows the given tab (and hides all others), also marks submenu entries
   * @param {Number} index What tab should be displayed [1...tabCount]
   */
  this.showTab = function (index) {
    this.setStatus("Loading tab...", true, true);
    if (document.getElementById("LEDLegend") != null) document.getElementById("LEDLegend").style.display = "none";
    for (var i = 1; i <= this.tabCount; i++) {
      document.getElementById("Tab" + i.toString()).style.display = "none";
    }
    if (document.getElementById("Tab" + index) != null) document.getElementById("Tab" + index).style.display = "";
    ResponsiveDynamic.toggleMenu(false);
    if (this.tabCount > 1) {
      var elements = document.getElementById("NavFrame").getElementsByTagName("DIV");
      for (var i = 0; i < elements.length; i++) {
        if (elements[i].className == "Menu_DropDown") {
          var links = elements[i].getElementsByTagName("A");
          if (links[0].href.substring(0, 11) == "javascript:") {
            for (var j = 0; j < links.length; j++) {
              links[j].style.fontWeight = "";
              links[j].style.backgroundColor = "";
              links[j].style.color = "";
              if (j == window.parseInt(index, 10) - 1) {
                links[j].style.color = "White";
                links[j].style.fontWeight = "bold";
                links[j].style.backgroundColor = "#A0A0A0";
              }
            }
          }
        }
      }
      elements = document.getElementById("NavMenu").getElementsByTagName("DIV");
      for (var i = 0; i < elements.length; i++) {
        links = elements[i].getElementsByTagName("A");
        if (links[1].href.substring(0, 11) == "javascript:") {
          for (var j = 0; j < links.length; j++) {
            links[j].style.fontWeight = "";
            links[j].style.backgroundColor = "";
            if (j == window.parseInt(index, 10)) {
              links[j].style.fontWeight = "bold";
              links[j].style.backgroundColor = "White";
            }
          }
        }
      }
    }
    if (this.constructor.resize != null) this.constructor.resize();
    this.setStatus("", false, false);
  }
  /**
   * @description This method sets the high of all tabs (needed within responsive stuff)
   * @param {String} height CSS Style height content
   */
  this.setTabHeight = function (height) {
    for (var i = 1; i <= this.tabCount; i++) {
      document.getElementById("Tab" + i.toString()).style.height = height;
    }
  }
  /**
   * @description This method initializes module visualizations (completely generates IFRAME content)
   * @param {HTMLDOM-iframe} iframe The IFRAME, that should be used
   * @param {String} view Visualization type [front, side]
   * @param {Object} startValues Initial display values
   */
  this.initModule = function (iframe, view, startValues) {
    iframe.contentWindow.dynamic = new ModulDynamic(iframe.contentWindow.document);
    switch (this.moduleIdentifier["Style"]) {
      case "B":
        iframe.contentWindow.dynamic.init("rod-" + view, this.moduleIdentifier["Network"]);
        if (view == "front") {
          iframe.contentWindow.dynamic.updateLEDs(startValues);
          iframe.style.width = "250px";
          iframe.style.height = "313px";
        } else if (view == "side") {
          iframe.contentWindow.dynamic.updateMagnets("rod", window.parseInt(this.moduleIdentifier["Length"], 10), window.parseInt(startValues["Count"], 10), startValues["PosRaw"]);
          iframe.style.width = "100px";
          iframe.style.height = "906px";
        }
        break;
      case "P":
        iframe.contentWindow.dynamic.init("profile-" + view, this.moduleIdentifier["Network"]);
        if (view == "front") {
          iframe.contentWindow.dynamic.updateLEDs(startValues);
          iframe.style.width = "250px";
          iframe.style.height = "267px";
        } else if (view == "side") {
          iframe.contentWindow.dynamic.updateMagnets("profile", window.parseInt(this.moduleIdentifier["Length"], 10), window.parseInt(startValues["Count"], 10), startValues["PosRaw"]);
          iframe.style.width = "100px";
          iframe.style.height = "673px";
        }
        break;
      default:
    }
  }
  /**
   * @description This method highlights the current page within the navogation bar
   * @param {String} site Current path, corresponding location.pathname
   */
  this.mark = function (site) {
    switch (site.split("/")[site.split("/").length - 1].toLowerCase()) {
      case "":
      case "index.htm":
        document.getElementById("NavFrame").children[4].className = "Symbols_right Marked";
        document.getElementById("NavMenu").getElementsByTagName("A")[0].className = "Menu_Entry_Marked";
        break;
      case "user.htm":
        document.getElementById("NavFrame").children[3].className = "Symbols_right Marked";
        document.getElementById("NavMenu").getElementsByTagName("A")[4].className = "Menu_Entry_Marked";
        break;
      case "setup.htm":
        document.getElementById("NavFrame").children[2].className = "Symbols_right Marked";
        document.getElementById("NavMenu").getElementsByTagName("A")[2].className = "Menu_Entry_Marked";
        break;
      case "log.htm":
        document.getElementById("NavFrame").children[1].className = "Symbols_right Marked";
        document.getElementById("NavMenu").getElementsByTagName("A")[8].className = "Menu_Entry_Marked";
        break;
      case "info.htm":
        document.getElementById("NavFrame").children[0].className = "Symbols_right Marked";
        document.getElementById("NavMenu").getElementsByTagName("A")[9].className = "Menu_Entry_Marked";
        break;
    }
  }
  /**
   * @description This method handles authentification related display elements (navigation bar / authentification pop up)
   * @param {Object} values Containing AuthState status
   */
  this.updateMenu = function (values) {
    if (document.getElementById("AuthText1").parentElement.getElementsByTagName("OBJECT").length > 0) var vecGraphicsUser = document.getElementById("AuthText1").parentElement.getElementsByTagName("OBJECT")[0].contentDocument;
    else var vecGraphicsUser = null;
    if (values["AuthState"] == "LoggedIn") {
      if (vecGraphicsUser != null) vecGraphicsUser.getElementsByTagName("text")[0].textContent = "-";
      document.getElementById("AuthText1").innerHTML = "Logout";
      document.getElementById("AuthText2").innerHTML = "Logout";
      this.prepareAuthPopUp(true);
    } else {
      if (vecGraphicsUser != null) vecGraphicsUser.getElementsByTagName("text")[0].textContent = "+";
      document.getElementById("AuthText1").innerHTML = "Login";
      document.getElementById("AuthText2").innerHTML = "Login";
      this.prepareAuthPopUp(false);
    }
  }
  /**
   * @description This method sets the page icon
   * @param {String} url Icon file
   */
  this.setIcon = function (url) {
    if (document.getElementById("PageIcon") != null) {
      var icon = document.getElementById("PageIcon");
      if (icon.href.split("/")[icon.href.split("/").length - 1] != url.split("/")[url.split("/").length - 1]) {
        if (document.head != null) document.head.removeChild(document.getElementById("PageIcon"));
        else document.getElementsByTagName("HEAD")[0].removeChild(document.getElementById("PageIcon"));
        icon = document.createElement("LINK");
        icon.id = "PageIcon";
        icon.rel = "icon";
        icon.href = url;
        icon.type = "image/x-icon";
        if (document.head != null) document.head.appendChild(icon);
        else document.getElementsByTagName("HEAD")[0].appendChild(icon);
      }
    } else {
      var icon = document.createElement("LINK");
      icon.id = "PageIcon";
      icon.rel = "icon";
      icon.href = url;
      icon.type = "image/x-icon";
      if (document.head != null) document.head.appendChild(icon);
      else document.getElementsByTagName("HEAD")[0].appendChild(icon);
    }
  }
  /**
   * @description This method modifies the authentification popup to show login or logout dialog
   * @param {Boolean} type Login dialog (false) or logout dialog (true)
   */
  this.prepareAuthPopUp = function (type) {
    if (type) {
      var authTable = document.getElementById("AuthTable");
      authTable.rows[0].cells[1].children[0].disabled = true;
      authTable.rows[0].style.display = "none";
      authTable.rows[1].cells[1].children[0].disabled = true;
      authTable.rows[1].style.display = "none";
      authTable.parentElement.children[0].value = "logout"
      authTable.rows[2].cells[1].children[0].value = "Logout";
    } else {
      var authTable = document.getElementById("AuthTable");
      authTable.rows[0].cells[1].children[0].disabled = false;
      authTable.rows[0].style.display = "";
      authTable.rows[1].cells[1].children[0].disabled = false;
      authTable.rows[1].style.display = "";
      authTable.parentElement.children[0].value = "login"
      authTable.rows[2].cells[1].children[0].value = "Login";
    }
  }
  /**
   * @description This method toggles the display of the authentification pop up
   */
  this.toggleAuthPopUp = function () {
    if (document.getElementById("AuthTable").parentElement.parentElement.style.display == "") document.getElementById("AuthTable").parentElement.parentElement.style.display = "none";
    else document.getElementById("AuthTable").parentElement.parentElement.style.display = "";
  }
  /**
  * @description This method handles incoming content of IFRAMES after something has been posted
  * @param {HTMLDOM-iframe} iframe The corresponding IFRAME (FORM target)
  */
  this.postIncome = function (iframe) {
    if(iframe.src != "") {
      this.update(iframe.contentWindow.startValues);
    }
  }
  /**
  * @description This method calls acyclic background HTTP POSTs or GETs
  * @param {String} filename Path to the target file serverside
  * @param {String} method HTTP type [POST, GET]
  * @param {Object} payload Some optional content for HTTP POSTs
  * @param {Function} fallback Function to be called onReadyStateChange
  */
  this.load = function (filename, method, payload, fallback) {
    var httpRequest = new XMLHttpRequest();
    if (method.toUpperCase() == "GET" || method.toUpperCase() == "POST") {
      httpRequest.open(method.toUpperCase(), filename, true);
      httpRequest.onreadystatechange = function () { fallback(httpRequest) };
      httpRequest.send(payload);
    }
  }
  /**
  * @description This method handles finished XMLHTTPRequests
  * @param {Object} httpReq XmlHttpRequest object
  */
  this.prepareUpdate = function (httpReq) {
    if (httpReq.readyState == 4 && httpReq.status == 200) {
      dynamic.update(window.eval("(" + httpReq.responseText + ")"));
    } else if (httpReq.readyState == 4 && httpReq.status != 200) {
      this.setStatus("Update error: HTTP " + httpReq.statusText, false, true);
    }
  }
}
/**
 * @description This method checks if some input is a valid IP adress syntax
 * @param {HTMLDOM-input} input IP adress input field to be checked
 */
PageDynamic.checkIP = function (input) {
  if (input.priorValue == null) input.priorValue = "";
  var octets = input.value.split(".");
  if (octets.length < 5) {
    for (var i = 0; i < octets.length; i++) {
      if (octets[i] != "") {
        var octet = window.parseInt(octets[i], 10);
        if (window.isNaN(octet) || octets[i].match(/\D/)) {
          input.value = input.priorValue;
          return octets[i] + " is not a Number.";
        } else if (octet > 255 || octet < 0) {
          input.value = input.priorValue;
          return octet.toString() + " is out of range [0 - 255].";
        }
      }
    }
    input.priorValue = input.value;
    return "valid";
  } else {
    input.value = input.priorValue;
    return octets.length.toString() + " where too many octets.";
  }
};
/**
 * @description This method simply adds table cells containing a given text to a table row
 * @param {HTMLDOM-row} row The row to be extended
 * @param {Array} cells An array of Strings, each element will be added as single cell
 * @returns {HTMLDOM-row} The new table row containing all cells
 */
PageDynamic.addCellsToRow = function (row, cells) {
  for (var i = 0; i < cells.length; i++) row.insertCell(-1).innerHTML = cells[i];
  return row;
}
/**
 * @description This method creates a input specified by several elements
 * @param {String} id Optional an id for this element
 * @param {String} type Optional an input type for this element
 * @param {String} name Optional a name for this element
 * @param {String} value Optional a value for this element
 * @param {Boolean} disabled Optional if this element should be disabled
 * @returns {HTMLDOM-input} The newly generated input field
 */
PageDynamic.createInput = function (id, type, name, value, disabled) {
  var input = document.createElement("INPUT");
  if (id != null) input.id = id;
  try {
    if (type != null) input.type = type;
  } catch (exception) {
    if (type != null) input.type = "text";
  }
  if (name != null) input.name = name;
  if (value != null) input.value = value;
  if (disabled != null) input.disabled = disabled;
  return input;
}